# -*- coding: utf-8 -*-
from django.conf import settings

PROPRIETARY_APPLICATION_NAME = getattr(settings, 'PROPRIETARY_APPLICATION_NAME', "Owner")
PROPRIETARY_BACKEND_NAME = getattr(settings, 'PROPRIETARY_BACKEND_NAME', "Django")



# -*- coding: utf-8 -*-

from django.core.urlresolvers import reverse

from rest_framework.authentication import BaseAuthentication, get_authorization_header
from rest_framework import exceptions

from social.apps.django_app.views import NAMESPACE
from social.apps.django_app.utils import load_backend, load_strategy
from social.exceptions import MissingBackend


class SocialAuthentication(BaseAuthentication):
    """
    Authentication backend using `python-social-auth`

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header with the backend used, prepended with the string "Bearer ".

    For example:

        Authorization: Bearer facebook 401f7ac837da42b97f613d789819ff93537bee6a
    """
    www_authenticate_realm = 'api'

    def authenticate(self, request):
        """
        Returns two-tuple of (user, token) if authentication succeeds,
        or None otherwise.
        """

        auth = get_authorization_header(request).split()

        if not auth or auth[0].lower() != b'bearer':
            return None

        if len(auth) == 1:
            msg = 'Invalid token header. No backend provided.'
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) == 2:
            msg = 'Invalid token header. No credentials provided.'
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 3:
            msg = 'Invalid token header. Token string should not contain spaces.'
            raise exceptions.AuthenticationFailed(msg)

        token = auth[2]
        backend = auth[1]

        strategy = load_strategy(request=request)

        try:
            backend = load_backend(strategy, backend, reverse(NAMESPACE + ":complete", args=(backend,)))
        except MissingBackend:
            msg = 'Invalid token header. Invalid backend.'
            raise exceptions.AuthenticationFailed(msg)

        user = backend.do_auth(access_token=token)

        if not user:
            msg = 'Bad credentials.'
            raise exceptions.AuthenticationFailed(msg)
        return user, token

    def authenticate_header(self, request):
        """
        Bearer is the only finalized type currently
        """
        return 'Bearer backend realm="%s"' % self.www_authenticate_realm



# -*- coding: utf-8 -*-

from django.conf.urls import url, patterns, include
from oauth2_provider.views import AuthorizationView, TokenView, RevokeTokenView

from .views import convert_token

urlpatterns = patterns(
    '',
    url(r'^authorize/?$', AuthorizationView.as_view(), name="authorize"),
    url(r'^token/?$', TokenView.as_view(), name="token"),
    url('', include('social.apps.django_app.urls', namespace='social')),
    url(r'^convert-token/?$', convert_token, name="convert_token"),
    url(r'^revoke-token/?$', RevokeTokenView.as_view(), name="revoke_token"),
)


# -*- coding: utf-8 -*-

from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.response import Response
from rest_framework import status
from rest_framework import permissions

from oauth2_provider.models import Application, AccessToken, RefreshToken
from oauthlib.common import generate_token

from .authentication import SocialAuthentication
from .settings import PROPRIETARY_APPLICATION_NAME

from django.utils import timezone
from datetime import timedelta

@api_view(['GET'])
@authentication_classes([SocialAuthentication])
@permission_classes([permissions.IsAuthenticated])
def convert_token(request):
    try:
        app = Application.objects.get(name=PROPRIETARY_APPLICATION_NAME)
    except Application.DoesNotExist:
        return Response({
            "detail": "The server's oauth2 application is not setup or misconfigured"
        }, status=status.HTTP_403_FORBIDDEN)

    code = status.HTTP_200_OK

    token = AccessToken.objects.filter(user=request.user, application=app).order_by('expires').last()
    # If the token is goinf to expire within the minute, we generate a new one
    if not token or int((token.expires - timezone.now()).total_seconds()) < 60:
        token = AccessToken.objects.create(user=request.user, application=app,
            token=generate_token(), expires=timezone.now() + timedelta(days=1),
            scope="read write")
        refresh_token = RefreshToken.objects.create(access_token=token,
            token=generate_token(), user=request.user, application=app)
        code = status.HTTP_201_CREATED
    else:
        try:
            refresh_token = RefreshToken.objects.get(access_token=token,
                user=request.user, application=app)
        except RefreshToken.DoesNotExist:
            refresh_token = RefreshToken.objects.create(access_token=token,
                token=generate_token(), user=request.user, application=app)

    return Response({
        "access_token": token.token,
        "refresh_token": refresh_token.token,
        "token_type": "Bearer",
        "expires_in": int((token.expires - timezone.now()).total_seconds()),
        "scope": token.scope
    }, status=code)



# -*- coding: utf-8 -*-

from social.backends.oauth import BaseOAuth2
from django.core.urlresolvers import reverse
from .settings import PROPRIETARY_BACKEND_NAME

class DjangoOAuth2(BaseOAuth2):
    """Default OAuth2 authentication backend used by this package"""
    name = PROPRIETARY_BACKEND_NAME
    AUTHORIZATION_URL = reverse('authorize')
    ACCESS_TOKEN_URL = reverse('token')